﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LINQ_Example.Data;
using System.Xml.Schema;
using System.Xml;
using System.Xml.Linq;

namespace LINQ_Example
{
    /// <summary>
    /// This example shows various uses of LINQ
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            /** 
             * LINQ to Objects
             */
            Student[] students = new Student[5];
            for (int i = 0; i < 5; i++)
            {
                students[i] = new Student()
                {
                    Name = "Arnošt" + i,
                    Age = 20 + i,
                    StudentsBrain = new Student.Brain() { IQ = 120 + i }
                };

            }
            // smartStudents is of type: IEnumerable<Student>
            var smartStudentNames = from s in students 
                                    where s.StudentsBrain.IQ > 122 
                                    select s.Name;

            foreach (var item in smartStudentNames)
            {
                Console.WriteLine(item);
            }


            /** 
             * LINQ to SQL 
             */
            var db = new DataClasses1DataContext();

            var cheapProducts = from p in db.PRODUCTs
                                where p.unit_price < 22
                                select p;

            foreach (var item in cheapProducts)
            {
                Console.WriteLine("Name: {0}, Price: {1}", item.name, item.unit_price);
            }


            /**
             * XML Validation
             */ 
            string xml = @"..\..\Data\data.xml";
            string xsd = @"..\..\Data\schema.xsd";

            // Create the XmlSchemaSet class.
            XmlSchemaSet sc = new XmlSchemaSet();

            // Add the schema to the collection.
            sc.Add("", xsd);

            // Set the validation settings.
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.ValidationType = ValidationType.Schema;
            settings.Schemas = sc;

            // Create the XmlReader object.
            XmlReader reader = XmlReader.Create(xml, settings);

            // Parse the file - throws an exception, if the XML is not valid
            while (reader.Read());


            /** 
             * LINQ to XML 
             */
            XDocument doc = XDocument.Load(xml);

            var seriesWith5Stars = from series in doc.Root.Elements("Series")
                                   where Convert.ToInt32(series.Element("Stars").Value) == 5
                                   select new
                                   {
                                       Title = series.Element("Title").Value,
                                       Stars = series.Element("Stars").Value
                                   };

            foreach (var item in seriesWith5Stars)
            {
                Console.WriteLine("Title: {0}, Nr. of stars: {1}", item.Title, item.Stars);
            } 

            Console.ReadKey();
        }
    }

    /// <summary>
    /// A complex object that can be traversed using LINQ
    /// </summary>
    class Student
    {
        public string Name { get; set; }
        public int Age { get; set; }

        public Brain StudentsBrain { get; set; }

        public class Brain
        {
            public int IQ { get; set; }
        }

    }

}
